# -*- coding: utf-8 -*-

from .user_entities_request import UserEntity
from .user_entities_request import UserEntityEntry
from .user_entities_request import UserEntitiesRequest

"""
apiai
~~~~~~~~~~~~~~~~
This module provides a ApiAI classes to manage requests.
"""

__all__ = [
    'UserEntity',
    'UserEntityEntry',
    'UserEntitiesRequest'
]
